#!/usr/bin/perl
#
# This script will convert a dictionary file in Sun (INI) format into
# a .h file for packet-diameter.c
#
# We need to add #defines for CommandCodes and AvpCodes.
#
# We need to build:
#     diameter_vals               (CommandCode -> string)
#     diameter_printinfo          (AvpCode -> DataType)
#     diameter_attrib_type_vals   (AvpCode -> string)              
#

my $USAGE="USAGE: $0 [-o outputFile] inputFile\n";

my $outputFile="packet-diameter.h";
my $inputFile;

use vars qw($opt_o);
use Getopt::Std;

my @attDefs;
my @attStrs;
my @attTypes;
my @valArray; # temporary intermediate storage
my @valuesArray; # holds all values

#
# This routine will parse in a diameter INI style dictionary
#
sub parseDictionary($;) {
  my $inputFile = shift;
  my $AVP;
  my $rawAVP;
  my $VALUE;

  open(IN, "<$inputFile") || die "Unable to open $inputFile ($!)\n";

  while (<IN>) {
    chomp $_;
#    print "Line: $_\n";
    s/([^\\])\#.*$/$1/; # Remove Comments
    s/^\#.*$/$1/; # Remove Comments
#    print "Line (comments removed): $_\n";
    next if (/^\s*$/); # Blank line

    # Ok, we have some data
#    print "Data: $_\n";

    # Remove leading and trailing white space
    s/^\s*(.*)\s*$/$1/;

    # If it is a section, set our section type and read in the data.
    if (/^\[/) {
      # We have a Section!
      ($AVP)=/\[\s*AVP\s*([^\]]*)\]/;
      if (defined($AVP)) {
	undef $VALUE;
	$rawAvp = $AVP;
	$AVP =~	s/^\s*(.*)\s*$/$1/;
	# Convert the AVP to something more #definable readable
	$AVP =~ tr/a-z/A-Z/;
	$AVP =~ s/[.-]/_/g;
#	print "Found AVP $AVP\n";
      } else {
	($VALUE)=/\[\s*VALUE\s*([^\]]*)\]/;
	if (defined($VALUE)) {
	  undef $AVP;
	  # First, dump out previous value's array
	  if (defined(@valArray)) {
	    push @valArray, "   {0, (char *)NULL}\n";
	    push @valuesArray, "\n";
	    push @valuesArray, @valArray;
	    push @valuesArray, "};\n";
	    undef @valArray;
	  }
	  $VALUE =~ s/^\s*(.*)\s*$/$1/;
	  $VALUE =~ tr/A-Z/a-z/;
	  $VALUE =~ s/[.-]/_/g;
#	  print "Found VALUE $VALUE!\n";
	}
      }
    } else { # not a section, so must be a label
      # If we find a value, output our line
      if (defined($AVP)) {
	my ($val) = /^Value\s*\=\s*(.*)$/;
	if (defined ($val)) {
	  my $str = "#define DIAMETER_ATT_$AVP $val\n";
	  push @attDefs, $str;
	  push @attStrs, "{DIAMETER_ATT_$AVP, \"$rawAvp\"},\n";
	} else {
	  my ($type) = /^Type\s*\=\s*(.*)$/;
	  if (defined ($type)) {
	    $type = "DIAMETER_STRING" if ($type eq "string");
	    $type = "DIAMETER_DATA" if ($type eq "data");
	    $type = "DIAMETER_ADDRESS" if ($type eq "address");
	    $type = "DIAMETER_INTEGER32" if ($type eq "integer32");
	    $type = "DIAMETER_INTEGER64" if ($type eq "integer64");
	    $type = "DIAMETER_TIME" if ($type eq "time");
	    $type = "DIAMETER_COMPLEX" if ($type eq "complex");
	    my $str = "{ DIAMETER_ATT_$AVP, $type },\n";
	    push @attTypes, $str;
	  }
	}
      } else { # Maybe a value?
	if (defined($VALUE)) {
	  my ($val,$code) = /^(\S*)\s*\=\s*(.*)$/;
	  if (defined ($val) && defined ($code)) {
	    if (!defined(@valArray)) {
	      push @valArray, "static value_string diameter_" . $VALUE .
		"_vals\[\]={\n";
	    }
	    push @valArray, "   {$code, \"$val\"},\n";
	  }
	} # a value
      }
    } # We had a label
  } # While <IN>

  close IN;

  if (defined @valArray) {
    push @valArray, "   {0, (char *)NULL}\n";
    push @valuesArray, "\n";
    push @valuesArray, @valArray;
    push @valuesArray, "};\n";
    undef @valArray;
  }
  
} # parseDictionary

# This function will print each element of the array indented with
# the second argument
sub indprint($;$;@;) {
  my $fh = shift;
  my $ind = shift;
  my @array = @_;
  my $ele;

  foreach $ele (@array) {
    print $fh $ind . $ele;
  }
  
} # indprint


# HEADERS AND TRAILERS

my $HEADER="/*
 * Filename: packet-diameter.h
 * WARNING:  This file was automatically generated by dict2h.pl.  Modifications
 *           will be erased by next invocation of dictionary parser.
 */

#ifndef _PACKET_DIAMETER_H
#define _PACKET_DIAMETER_H
";

my $ATT_DEFS_HEADER="
/* Attributes */
";

my $ATT_DEFS_TRAILER="\n\n";

my $ATT_STRS_HEADER="            /* Attribute to String table */

static value_string diameter_attrib_type_vals[] = {
";
my $ATT_STRS_TRAILER="   {0, (char *)NULL}\n};\n\n\n";

my $ATT_TYPES_HEADER="           /* Attribute to Type Table */

static value_value_pair diameter_printinfo[] = {
";

my $ATT_TYPES_TRAILER = "   {0, 0}\n};\n\n\n";

my $VALUES_HEADER = "\n\n        /* Values Structures */\n";

my $TRAILER="\n\n#endif /* _PACKET_DIAMETER_H */\n";

# This routine will write our output
sub writeOutput($;) {
  my $fileName=shift;

  open (OUT, ">$fileName") || die "Unable to open $fileName ($!)\n";

  print OUT $HEADER;

  print OUT $ATT_DEFS_HEADER;
  print OUT @attDefs;
  print OUT $ATT_DEFS_TRAILER;

  print OUT $ATT_STRS_HEADER;
  indprint(\*OUT, "   ", @attStrs);
  print OUT $ATT_STRS_TRAILER;
  
  print OUT $ATT_TYPES_HEADER;
  indprint(\*OUT, "   ", @attTypes);
  print OUT $ATT_TYPES_TRAILER;
  
  print OUT $VALUES_HEADER;
  print OUT @valuesArray;
  print OUT $VALUES_TRAILER;

  print OUT $TRAILER;

  close OUT;

} #writeOutput











getopt('o:');

$outputFile = $opt_o if (defined($opt_o));
$inputFile = $ARGV[0];

die "$USAGE" if (!defined($inputFile));

#
# Ok, our filename is set
#

print "Converting $inputFile -> $outputFile \n";


parseDictionary($inputFile);

writeOutput($outputFile);
